//@author: Digital Slaves
//@help: Charcoal style
//@tags: Charcoal, Line, Contour

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float2 TextRes = float2(256,256);
float width <String uiname="Line Width";> = 2;
float4 BC : COLOR <String uiname="Background Color";>  = {1, 1, 1, 1};
float4 LC : COLOR <String uiname="Line Color";>  = {1, 1, 1, 1};
float Alpha ;

texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
	AddressU  = CLAMP;
    AddressV  = CLAMP;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: vertexshader to pixelshader
//used as output data with the VS function
//and as input data with the PS function

struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
	
	float a = width/TextRes.x;
	float b = width/TextRes.y;
   
	float4 c1 = tex2D(Samp, In.TexCd);
	float4 c2 = tex2D(Samp, In.TexCd + float2(a,0));
	float4 c3 = tex2D(Samp, In.TexCd + float2(0,b));
    
	float f = 0;
	
	f += abs(c1.x-c2.x);
	f += abs(c1.y-c2.y);
	f += abs(c1.z-c2.z);
    
	f += abs(c1.x-c3.x);
	f += abs(c1.y-c3.y);
	f += abs(c1.z-c3.z);
    
	f -= 0.2;  
 
	f = saturate(f);  

	c1.rgb = BC.rgb * (1-f) + LC.rgb * f;
	c1.a *= Alpha;
	
	return c1;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TCharcoal
{
    pass P0
    {
        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_1_0 PS();
    }
}

