//@author: Digital Slaves
//@help: Hatch style
//@tags: Hatch, tonal

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float4 DC : COLOR <String uiname="Diffuse Color";>  = {1, 1, 1, 1};
float Scale = 8;
float Brightness = 1.0;

//Texture
texture Tex <string uiname="Texture to Hatch";>;
sampler Samp = sampler_state
{
	Texture = <Tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU  = Wrap;
	AddressV  = Wrap;
};

texture Tex1 <string uiname="Texture Hatch Style";>;
sampler Samp1 = sampler_state
{
	Texture = <Tex1>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU  = Wrap;
	AddressV  = Wrap;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: vertexshader to pixelshader
//used as output data with the VS function
//and as input data with the PS function

struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0,
    float3 diffuse: COLOR,
    float3 normal: NORMAL)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}


// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    float4 col = tex2D(Samp, In.TexCd) ;
	
    float  c =(col.r + col.g + col.b) / 3.0 ;
	
	c *= Brightness;
	
	float3 col2 = tex2D(Samp1, In.TexCd * Scale);
	
	float g0 = 0;
	float g1 = 0;
	float f  = 0;
	
	if( c >= 0.666)
	{
		f =(c - 0.666) / 0.333;
		g1 = 0;
		g0 = col2.r;
	}
	else if( c >= 0.333)
	{
		f =(c - 0.333) / 0.333;
		g1 = col2.r;
		g0 = col2.g;
	}
	else{
		f = c / 0.333;
		g1= col2.g;
		g0= col2.b;
	}
	
	float g = g0 + f *(g1 - g0);
	
	g = 1 - g;
  
    col = float4( g ,g ,g ,col.a ) * DC;
	
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique THatching
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_1_0 PS();
    }
}

